/* SPDX-License-Identifier: GPL-2.0 */
/*
 * (C) Copyright 2020 Xilinx, Inc.
 * Michal Simek <michal.simek@xilinx.com>
 */

#ifndef _BOARD_XILINX_COMMON_BOARD_H
#define _BOARD_XILINX_COMMON_BOARD_H

int board_late_init_xilinx(void);

int xilinx_read_eeprom(void);

#endif /* BOARD_XILINX_COMMON_BOARD_H */
