/* SPDX-License-Identifier:    GPL-2.0 */
/*
 * Copyright (C) 2018, 2019 Marvell International Ltd.
 */

#ifndef __CLOCK_H__

/** System PLL reference clock */
#define PLL_REF_CLK                     50000000        /* 50 MHz */
#define NS_PER_REF_CLK_TICK             (1000000000 / PLL_REF_CLK)

#endif /* __CLOCK_H__ */
